HEX_ARR:
.db	"0123456789ABCDEF"

LCD_Write:	;Write byte to LCD - 114 cycles
	push	R17

	cbi	LCD_PORT,LCD_SCE
	ldi	R17,8
	LCD_Write_loop:
		cbi	LCD_PORT,LCD_SCLK
		cbi	LCD_PORT,LCD_DO
		sbrc	Input,7
		sbi	LCD_PORT,LCD_DO
		sbi	LCD_PORT,LCD_SCLK
		lsl	Input
		dec	R17
	brne	LCD_Write_loop

	pop	R17
ret

LCD_Set_X:	;Set Cursor X
	cbi	LCD_PORT,LCD_DC
	andi	Input,0b01111111
	ori	Input,0b10000000
	rcall	LCD_Write
	sbi	LCD_PORT,LCD_DC
ret

LCD_Set_Y:	;Set Cursor Y
	cbi	LCD_PORT,LCD_DC
	andi	Input,0b00001111
	ori	Input,0b01000000
	rcall	LCD_Write
	sbi	LCD_PORT,LCD_DC
ret

LCD_Set_H:	;Set horizontal record
	push	Input
	cbi	LCD_PORT,LCD_DC
	ldi	Input,0b00100000
	rcall	LCD_Write
	sbi	LCD_PORT,LCD_DC
	pop	Input
ret

LCD_Set_V:	;Set vertical record
	push	Input
	cbi	LCD_PORT,LCD_DC
	ldi	Input,0b00100010
	rcall	LCD_Write
	sbi	LCD_PORT,LCD_DC
	pop	Input
ret

LCD_Init:	;Init display logic
	sbi	LCD_PORT-1,LCD_SCLK
	sbi	LCD_PORT-1,LCD_DO
	sbi	LCD_PORT-1,LCD_DC
	sbi	LCD_PORT-1,LCD_SCE
	sbi	LCD_PORT-1,LCD_RES
	

	cbi	LCD_PORT,LCD_DC
	sbi	LCD_PORT,LCD_SCE
	cbi	LCD_PORT,LCD_RES
	rcall	Delay
	sbi	LCD_PORT,LCD_RES
	rcall	Delay

	cbi	LCD_PORT,LCD_SCE
	cbi	LCD_PORT,LCD_DC
	ldi	Input,0x21
	rcall	LCD_Write
	ldi	Input,0x11
	rcall	LCD_Write
	ldi	Input,0x9C
	rcall	LCD_Write
	ldi	Input,0x20
	rcall	LCD_Write
	ldi	Input,0x0C
	rcall	LCD_Write
	sbi	LCD_PORT,LCD_SCE

LCD_Clear:	;Cear display
	push	Input
	push	R17
	push	R18

	cbi	LCD_PORT,LCD_SCE
	sbi	LCD_PORT,LCD_DC
	
	ldi	R18,LCD_Height
	LCD_Clear_loopY:
		ldi	R17,LCD_Width
		LCD_Clear_loopX:
			clr	Input
			rcall	LCD_Write
			dec	R17
		brne	LCD_Clear_loopX
		dec	R18
	brne	LCD_Clear_loopY

	sbi	LCD_PORT,LCD_SCE

	cbi	LCD_PORT,LCD_DC
	ldi	Input,0b10000000
	rcall	LCD_Write
	ldi	Input,0b01000000
	rcall	LCD_Write

	pop	R18
	pop	R17
	pop	Input
ret

LCD_Write_Char:	;Write char to LCD
	push	ZH
	push	ZL
	push	R17
	sbi	LCD_PORT,LCD_DC
	ldi	ZH,$1
	mov	ZL,Input
	ldi	R17,6
	LCD_Write_Char_loop:
		lpm	Input,Z
		rcall	LCD_Write
		inc	ZH

		dec	R17
	brne	LCD_Write_Char_Loop
	pop	R17
	pop	ZL
	pop	ZH
ret

LCD_Write_Symbol:	;Write symbol (input*2, input*2+1 chars) to LCD
	push	ZH
	push	ZL
	push	R17
	sbi	LCD_PORT,LCD_DC
	lsl	Input
	push	Input
	ldi	R17,6
	ldi	ZH,$1
	mov	ZL,Input
	LCD_Write_Symbol_loop1:
		lpm	Input,Z
		rcall	LCD_Write
		inc	ZH

		dec	R17
	brne	LCD_Write_Symbol_loop1
	pop	Input
	inc	Input
	ldi	R17,6
	ldi	ZH,$1
	mov	ZL,Input
	LCD_Write_Symbol_loop2:
		lpm	Input,Z
		rcall	LCD_Write
		inc	ZH

		dec	R17
	brne	LCD_Write_Symbol_loop2
	pop	R17
	pop	ZL
	pop	ZH
ret


LCD_Write_Hex:	;Write hexadecimal number to LCD
	push	ZH
	push	ZL
	push	R17
	push	Input
	clr	R17
	swap	Input
	andi	Input,0x0F
	ldi	ZL,low(HEX_ARR*2)
	ldi	ZH,high(HEX_ARR*2)
	add	ZL,Input
	adc	ZH,R17
	lpm	Input,Z
	rcall	LCD_Write_Char
	pop	Input
	andi	Input,0x0F
	ldi	ZL,low(HEX_ARR*2)
	ldi	ZH,high(HEX_ARR*2)
	add	ZL,Input
	adc	ZH,Temp
	lpm	Input,Z
	rcall	LCD_Write_Char
	pop	R17
	pop	ZL
	pop	ZH
ret

LCD_Write_String:	;Write null-terminated string from PROGRAM memory
	push	Input
	LCD_Write_String_Loop:
		lpm	Input,Z+
		tst	Input
		breq	LCD_Write_String_Skip
		rcall	LCD_Write_Char
	rjmp	LCD_Write_String_Loop
	LCD_Write_String_Skip:
	pop	Input
ret

LCD_Write_String_Mem:	;Write null-terminated string from SRAM memory
	push	Input
	LCD_Write_String_Mem_Loop:
		ld	Input,Z+
		tst	Input
		breq	LCD_Write_String_Mem_Skip
		rcall	LCD_Write_Char
	rjmp	LCD_Write_String_Mem_Loop
	LCD_Write_String_Mem_Skip:
	pop	Input
ret

LCD_Draw_Progress_Bar:
	push	R17
	push	R18
	mov	R18,Input
	sbi	LCD_PORT,LCD_DC
	ldi	Input,0b00111110
	rcall	LCD_Write
	ldi	Input,0b01000001
	rcall	LCD_Write
	clr	R17
	LCD_Draw_Progress_Bar_Loop:
		ldi	Input,0b01000001
		cp	R18,R17
		brlo	LCD_Draw_Progress_Bar_No
		ldi	Input,0b01011101
		LCD_Draw_Progress_Bar_No:
		rcall	LCD_Write
		inc	R17
		cpi	R17,64
	brne	LCD_Draw_Progress_Bar_Loop
	ldi	Input,0b01000001
	rcall	LCD_Write
	ldi	Input,0b00111110
	rcall	LCD_Write
	pop	R18
	pop	R17
ret




