.include "m8def.inc"
.include "generic.h"
.include "LCD.h"
.include "MMC.h"
.include "Files.h"

;____________ VAR ___________________
.dseg
.org	0x060

DisRef:			.byte 1	; 
Volume:			.byte 1	;
PlayBufL:		.byte 256
PlayBufR:		.byte 256
PlayPos:		.byte 1
BufPos:			.byte 1
Prescaler:		.byte 1

;_________
Temp1:		.byte	1
;____________ PROG ___________________
.cseg
;Interrupts
.org	0x000	rjmp	Start
.org	0x009	rjmp	INT_Timer0


;Modules
.org	0x010
.include "chars.inc"
.include "sinus.inc"
.include "generic.inc"
.include "LCD.inc"
.include "MMC.inc"
.include "Files.inc"
.include "sound.inc"
.include "morse.inc"
;Strings
STR_Last_Update_Date:	.db	__DATE__,0
STR_Last_Update_Time:	.db	__TIME__,0

Start:
;_______ INIT __________
;Stack
ldi	R16,low(RAMEND)
out	SPL,R16
ldi	R16,high(RAMEND)
out	SPH,R16
;Key port

ldi	R16,(1<<Key1)|(1<<Key2)|(1<<Key3)|(1<<Key4)
out	KeyPort,R16

sbi	DDRB,1
sbi	DDRB,2
;Timer0
ldi	Temp,1		;Internal clock, no prescaler (~30KHz)
out	TCCR0,Temp	
ldi	Temp,1
out	TIMSK,Temp	;Interrupt enable
;Timer1
ldi	Temp,(1<<COM1A1)|(1<<COM1B1)|(1<<WGM10)
out	TCCR1A,Temp
ldi	Temp,(1<<WGM12)|(1<<CS10)
out	TCCR1B,Temp

ldi	Input,64		;
sts	Volume,Input

ldi	Input,10
sts	Morse_T,Input	;   * 114 
ldi	Input,6
sts	Morse_Len,Input	;  * 8,74 

sbi	DDRC,4

sei

;
rcall	LCD_Init
_LCD_Set_Pos_I	(LCD_Width-13*6)/2,LCD_Height/2
_LCD_Write_String_I	"My program!!!"
rcall	Delay_Long
; 
MMC_Init_Try:
rcall	MMC_Init
tst	Status

breq	MMC_Init_OK
	rcall	LCD_Clear
	_LCD_Write_String_I	"MMC: "
	cpi	Status,1<<MMC_Init_No_Card
	brne	MMC_Init_NoCard
		_LCD_Write_String_I	"no card"
	MMC_Init_NoCard:
	cpi	Status,1<<MMC_Init_Error
	brne	MMC_Init_InitError
		_LCD_Write_String_I	"init error"
	MMC_Init_InitError:
	rcall	Delay_Long
	rjmp	MMC_Init_Try
MMC_Init_OK:
rcall	LCD_Clear
; FAT
rcall	Files_Load_FAT
rcall	File_Find_First
;________ MAIN ________
MainLoop:
	rcall	LCD_Clear

	rcall	File_Write_Name
	_LCD_Set_Pos_I	0,1
	_LCD_Write_String_I	": "
	lds	Input,F_FileSize+0
	rcall	LCD_Write_Hex
	lds	Input,F_FileSize+1
	rcall	LCD_Write_Hex
	lds	Input,F_FileSize+2
	rcall	LCD_Write_Hex
	lds	Input,F_FileSize+3
	rcall	LCD_Write_Hex
	_LCD_Set_Pos_I	0,2
	_LCD_Write_String_I	": "
	lds	Input,F_FileAttr
	rcall	LCD_Write_Hex
	rcall	Delay_Long

	WaitKey:
		in	Temp,KeyPort-2
		sbrs	Temp,Key1
		rjmp	K1
		sbrs	Temp,Key2
		rjmp	K2
		sbrs	Temp,Key3
		rjmp	K3
		sbrs	Temp,Key4
		rjmp	K4
	rjmp	WaitKey
rjmp	MainLoop

K1:
	rcall	File_Find_Prev
rjmp	MainLoop

K2:
	rcall	File_Find_Next
rjmp	MainLoop

K3:
	lds	Temp,F_FileAttr
	sbrc	Temp,4
	rjmp	It_DIR

	_File_It_Ext	('P'),('I'),('C')
	brne	It_NO_PIC
	rjmp	It_PIC
	It_NO_PIC:

	_File_It_Ext	('T'),('X'),('T')
	brne	It_NO_TXT
	rjmp	It_TXT
	It_NO_TXT:


	_File_It_Ext	('R'),('A'),('W')
	brne	It_NO_RAW
	rjmp	It_RAW
	It_NO_RAW:

	_File_It_Ext	('M'),('O'),('R')
	brne	It_NO_MOR
	rjmp	It_MOR
	It_NO_MOR:
rjmp	MainLoop

It_DIR:
	clr	Input
	lds	Temp,F_File1Cluster+0
	cp	Temp,Input
	lds	Temp,F_File1Cluster+1
	cpc	Temp,Input
	brne	It_DIR_Not_Root

	lds	Temp,F_RootBase+0
	sts	F_DirOffset+0,Temp
	lds	Temp,F_RootBase+1
	sts	F_DirOffset+1,Temp
	lds	Temp,F_RootBase+2
	sts	F_DirOffset+2,Temp
	rcall	File_Find_First
	rjmp	MainLoop

	It_DIR_Not_Root:
	rcall	File_Reset_File
	rcall	File_Calc_Offset
	lds	Temp,F_FileOffset+0
	sts	F_DirOffset+0,Temp
	lds	Temp,F_FileOffset+1
	sts	F_DirOffset+1,Temp
	lds	Temp,F_FileOffset+2
	sts	F_DirOffset+2,Temp
	rcall	File_Find_First
rjmp	MainLoop


It_TXT:
	rcall	File_Reset_File
	TXT_Write_Loop:
	rcall	LCD_Clear
	clr	Temp
	TXT_Write_Loop_Y:
		inc	Temp
		clr	R18
		TXT_Write_Loop_X:
			rcall	File_It_End
			brsh	TXT_FileEnd
			rcall	File_Read_Byte_Buffer

			cpi	Input,13
			brne	No13
			tst	R18
			breq	TXT_Skip
			It13:
			cpi	R18,16
			brsh	TXT_Skip
			ldi	Input,' '
			inc	R18
			rcall	LCD_Write_Char
			rjmp	It13
			No13:

			cpi	Input,10
			breq	TXT_Skip

			cpi	Input,32
			brsh	No32
			ldi	Input,32
			No32:

			inc	R18
			rcall	CharFilter
			rcall	LCD_Write_Char
			
			TXT_Skip:
			cpi	R18,16
		brlo	TXT_Write_Loop_X
		ldi	input,' '
		rcall	LCD_Write_Char
		cpi	Temp,8
	brlo	TXT_Write_Loop_Y
	TXT_FileEnd:

	rcall	Delay_Long
	TXT_Wait:
	in	Temp,KeyPort-2
	sbrs	Temp,Key4
	rjmp	MainLoop
	sbrs	Temp,Key3
	rjmp	TXT_Write_Loop

	rjmp	TXT_Wait
rjmp	MainLoop

It_PIC:
	rcall	LCD_Clear
	rcall	File_Reset_File
	sbi	LCD_PORT,LCD_DC
	It_PIC_Loop:
		_LCD_Set_Pos_I	0,0
		ldi	Temp,LCD_Height-1
		PIC_Write_Loop_Y:
			ldi	R18,LCD_Width
			PIC_Write_Loop_X:
				rcall	File_Read_Byte_Buffer
				rcall	LCD_Write
				dec	R18
			brne	PIC_Write_Loop_X
			dec	Temp
		brne	PIC_Write_Loop_Y

		rcall	File_It_End
		brlo	PIC_FileNoEnd
		rcall	File_Reset_File
		PIC_FileNoEnd:

		sbis	KeyPort-2,Key4
		rjmp	MainLoop

		sbic	KeyPort-2,Key1
		rjmp	It_PIC_No1
		It_PIC_Prev_Loop:
		rcall	File_Find_Prev
		_File_It_Ext	('P'),('I'),('C')
		brne	It_PIC_Prev_Loop
		rcall	File_Reset_File
		rcall	LCD_Clear
		_LCD_Set_X_I	(LCD_Width-6-14*6)/2
		rcall	File_Write_Name
		rcall	Delay_Long
		rcall	Delay_Long
		It_PIC_No1:


		sbic	KeyPort-2,Key2
		rjmp	It_PIC_No2
		It_PIC_Next_Loop:
		rcall	File_Find_Next
		_File_It_Ext	('P'),('I'),('C')
		brne	It_PIC_Next_Loop
		rcall	File_Reset_File
		rcall	LCD_Clear
		_LCD_Set_X_I	(LCD_Width-6-14*6)/2
		rcall	File_Write_Name
		rcall	Delay_Long
		rcall	Delay_Long
		It_PIC_No2:


		rcall	Delay
		rcall	Delay
		rcall	Delay
		rcall	Delay
		rcall	Delay
		rcall	Delay
	rjmp	It_PIC_Loop
rjmp	MainLoop

It_RAW:
	rcall	LCD_Clear

	It_RAW_Pause_Loop0:
		in	Temp,KeyPort-2
		rcall	Delay
		sbrs	Temp,Key3
		rjmp	It_RAW_Pause_Loop0

	rcall	File_Reset_File
	_LCD_Set_Pos_I	((LCD_Width-6-6*14)/2),0
	rcall	File_Write_Name
	It_RAW_Loop:
		lds	Input,DisRef
		inc	Input
		andi	Input,15
		sts	DisRef,Input
		
		cpi	Input,5
		breq	It_RAW_Draw_Progress
		cpi	Input,10
		breq	It_RAW_Draw_Volume
		rjmp	It_RAW_No_Draw

		It_RAW_Draw_Progress:		
		_LCD_Set_Pos_I	(LCD_Width-6-68)/2,1
		lds	Arg1HH,F_FilePos+0
		lds	Arg1HL,F_FilePos+1
		lds	Arg1LH,F_FilePos+2
		lds	Arg1LL,F_FilePos+3
		clr	Arg2HH
		clr	Arg2HL
		clr	Arg2LH
		ldi	Temp,64
		mov	Arg2LL,Temp
		rcall	MUL32
		mov	Arg1HH,ResHH
		mov	Arg1HL,ResHL
		mov	Arg1LH,ResLH
		mov	Arg1LL,ResLL
		lds	Arg2HH,F_FileSize+0
		lds	Arg2HL,F_FileSize+1
		lds	Arg2LH,F_FileSize+2
		lds	Arg2LL,F_FileSize+3
		rcall	DIV32
		mov	Input,ResLL
		rcall	LCD_Draw_Progress_Bar

		It_RAW_Draw_Volume:
		_LCD_Set_Pos_I	(LCD_Width-6-68)/2,2
		lds	Input,Volume
		lsr	Input
		lsr	Input
		rcall	LCD_Draw_Progress_Bar

		It_RAW_No_Draw:

		rcall	File_Begin_Read_Sector
		clr	Temp
		It_RAW_Loop_Read:
			rcall	MMC_Read
			mov	Arg1LL,Input
			mov	Arg1LH,Input
			rcall	AddBuffer
			rcall	MMC_Read
			mov	Arg1LL,Input
			mov	Arg1LH,Input
			rcall	AddBuffer

			dec	Temp
		brne	It_RAW_Loop_Read
		rcall	File_End_Read_Sector

		in	Temp,KeyPort-2

		sbrc	Temp,Key3
		rjmp	It_RAW_No_Pause
		rcall	Delay
		It_RAW_Pause_Loop1:
			in	Temp,KeyPort-2
			rcall	Delay
			sbrs	Temp,Key3
			rjmp	It_RAW_Pause_Loop1

		It_RAW_Pause_Loop2:
			in	Temp,KeyPort-2
			rcall	Delay
			sbrc	Temp,Key3
			rjmp	It_RAW_Pause_Loop2

		It_RAW_Pause_Loop3:
			in	Temp,KeyPort-2
			rcall	Delay
			sbrs	Temp,Key3
			rjmp	It_RAW_Pause_Loop3
		It_RAW_No_Pause:

		sbrs	Temp,Key4
		rjmp	It_RAW_End

		lds	Input,Volume
		cpi	Input,0
		breq	It_RAW_Vol_0
		sbrs	Temp,Key1
		dec	Input
		It_RAW_Vol_0:
		cpi	Input,255
		breq	It_RAW_Vol_255
		sbrs	Temp,Key2
		inc	Input
		It_RAW_Vol_255:
		sts	Volume,Input

		rcall	File_It_End
		brsh	It_RAW_End
	rjmp	It_RAW_Loop
	It_RAW_End:
	
rjmp	MainLoop

It_MOR:
	rcall	LCD_Clear
	rcall	File_Reset_File
	It_MOR_Loop:
		rcall	File_It_End
		brsh	It_MOR_FileEnd

		_LCD_Set_Pos_I	(LCD_Width-6-68)/2,2
		lds	Input,Morse_Len
		lsl	Input
		rcall	LCD_Draw_Progress_Bar

		rcall	File_Read_Byte_Buffer

		mov	Temp,Input
		_LCD_Set_Pos_I	(LCD_Width-6-6)/2,0
		rcall	LCD_Write_Char
		mov	Input,Temp

		rcall	Morse_Code
		mov	Temp,Input
		_LCD_Set_Pos_I	(LCD_Width-6-6*5)/2,1
		rcall	Morse_Write
		mov	Input,Temp

		It_MOR_Loop_Repeat:
		mov	Temp,Input
		rcall	Morse_Play_Code
		mov	Input,Temp

		in	Temp,KeyPort-2

		sbrs	Temp,Key3
		rjmp	It_MOR_Loop_Repeat

		sbrs	Temp,Key4
		rjmp	It_MOR_End

		lds	Input,Morse_Len
		cpi	Input,1
		breq	It_MOR_Len_1
		sbrs	Temp,Key1
		dec	Input
		It_MOR_Len_1:
		cpi	Input,31
		brsh	It_MOR_Len_31
		sbrs	Temp,Key2
		inc	Input
		It_MOR_Len_31:
		sts	Morse_Len,Input

	rjmp	It_MOR_Loop
	It_MOR_FileEnd:
	It_MOR_End:

rjmp	MainLoop

AddBuffer:	;Arg1LL - Left; Arg1LH - Right
	push	Temp
	push	Input
	push	ZH
	push	ZL

	AddBuffer_Wait:
		lds	Temp,BufPos
		lds	Input,PlayPos
		cp	Temp,Input
	breq	AddBuffer_Wait

	ldi	ZH,high(PlayBufL)
	ldi	ZL,low(PlayBufL)
	lds	Temp,BufPos
	add	ZL,Temp
	clr	Temp
	adc	ZH,Temp

	lds	Input,Volume

	mov	Temp,Arg1LL
	subi	Temp,128
	mulsu	Temp,Input
	mov	Temp,R1
	subi	Temp,128
	st	Z,Temp

	inc	ZH

	mov	Temp,Arg1LH
	subi	Temp,128
	mulsu	Temp,Input
	mov	Temp,R1
	subi	Temp,128
	st	Z,Temp

	lds	Temp,BufPos
	inc	Temp
	sts	BufPos,Temp


	pop	ZL
	pop	ZH
	pop	Input
	pop	Temp
ret


K4:
	rcall	LCD_Clear
	rcall	Delay_Long
	K4_Loop:
		ldi	ZH,High(Sinus*2)
		subi	ZL,8
		lpm	R16,Z
		rcall	IR_Write_Byte
		rcall	Delay_Small

		sbic	PINB,Key4
	rjmp	K4_Loop
rjmp	MainLoop

IR_Write_0:					;Write logic 0
	sbi	PORTC,4
	rcall	Delay_Small
	cbi	PORTC,4
	rcall	Delay_Small
	rcall	Delay_Small
ret

IR_Write_1:					;Write logic 1
	sbi	PORTC,4
	rcall	Delay_Small
	rcall	Delay_Small
	cbi	PORTC,4
	rcall	Delay_Small
ret

IR_Write_Byte:				;R16 => IR
	cli
	push	R17
	rcall	IR_Write_1		;Start
	ldi	R17,8				;x8 bit
	IR_Write_Byte_Loop:
		sbrc	R16,7
		rcall	IR_Write_1	;bit=1
		sbrs	R16,7
		rcall	IR_Write_0	;bit=0
		lsl	R16
		dec	R17
	brne	IR_Write_Byte_Loop
	rcall	IR_Write_0		;End
	pop	R17
	sei
ret

Delay_Small_:	;20us
	push	R17
	ldi	R17,131
	Delay_Small_Loop_:
	dec	R17
	brne	Delay_Small_Loop_
	pop	R17
ret



INT_Timer0:
	push	ZH
	push	ZL
	push	Temp
	push	R18
	in	Temp,SREG
	push	Temp

	lds	Temp,Prescaler
	inc	Temp
;	cpi	Temp,3
;	brlo	INT_Timer0_No
;	clr	Temp
;	INT_Timer0_No:
	andi	Temp,3
	sts	Prescaler,Temp
	tst	Temp
	brne	INT_Timer0_Skip

	lds	Temp,PlayPos
	inc	Temp
	lds	R18,BufPos
	cp	Temp,R18
	breq	INT_Timer0_Skip
	sts	PlayPos,Temp

	ldi	ZH,high(PlayBufL)
	ldi	ZL,low(PlayBufL)
	lds	Temp,PlayPos
	add	ZL,Temp
	clr	Temp
	adc	ZH,Temp

	clr	Temp
	out	OCR1AH,Temp
	ld	Temp,Z
	out	OCR1AL,Temp

	inc	ZH
	clr	Temp
	out	OCR1BH,Temp
	ld	Temp,Z
	out	OCR1BL,Temp

	INT_Timer0_Skip:

	pop	Temp
	out	SREG,Temp
	pop	R18
	pop	Temp
	pop	ZL
	pop	ZH	
reti

.exit









